@@ -1,8 +1,8 @@
 from multiprocessing.pool import ApplyResult
 
 from explainaboard_api_client import ApiException
+import explainaboard_client
 from explainaboard_client.client import ExplainaboardClient
-from explainaboard_client.config import Config
 from explainaboard_client.tests.test_utils import TestEndpointsE2E
 
 
@@ -12,9 +12,10 @@ class TestUserAPI(TestEndpointsE2E):
         self.assertEqual(user["email"], "explainaboard@gmail.com")
 
     def test_401(self):
-        client = ExplainaboardClient(
-            Config("invalid_username", "invalid_api_key", "staging")
-        )
+        explainaboard_client.username = "invalid_username"
+        explainaboard_client.api_key = "invalid_api_key"
+        explainaboard_client.environment = "staging"
+        client = ExplainaboardClient()
         with self.assertRaises(ApiException):
             client.user_get()
 
