@@ -3,14 +3,8 @@ import pathlib
 from typing import Final
 from unittest import TestCase
 
+import explainaboard_client
 from explainaboard_client.client import ExplainaboardClient
-from explainaboard_client.config import Config
-
-TEST_CONFIG = Config(
-    "explainaboard@gmail.com",
-    "gEoVQz7pZzlmUR8sQiu6GQ",
-    "staging",
-)
 
 test_artifacts_path: Final = os.path.join(
     os.path.dirname(pathlib.Path(__file__)), "artifacts"
@@ -19,7 +13,10 @@ test_artifacts_path: Final = os.path.join(
 
 class TestEndpointsE2E(TestCase):
     def setUp(self):
-        self._client: ExplainaboardClient = ExplainaboardClient(TEST_CONFIG)
+        explainaboard_client.username = "explainaboard@gmail.com"
+        explainaboard_client.api_key = "gEoVQz7pZzlmUR8sQiu6GQ"
+        explainaboard_client.environment = "staging"
+        self._client = ExplainaboardClient()
 
     def tearDown(self) -> None:
         self._client.close()
